# -*- coding: utf-8 -*-
import json
import re

import requests as po
from .expand import (
    expandPoke,
    expandNudge,
    expandTemp,
    expandGetMessage,
    expandGetList,
    expandGetProFile,
    expandEventNewFriend,
    expandEventJoinGroup,
    expandEventBotJoinGroup,
    expandUploadImage,
    expandActionGroup,
    expandActionFriend,
    expandSetGroupConfig
)

from rich.console import Console

_config = {
    "send": {
        "friend": {
            "sendFriendMessage": "/sendFriendMessage"
        },
        "group": {
            "sendGroupMessage": "/sendGroupMessage"
        },
        "temp": {
            "sendTempMessage": "/sendTempMessage"
        },
        "recall": "/recall"
    },
    "get": {
        "info": "/about",
        "groupConfig": "/groupConfig"
    },
    "set": {
        "essence": "/setEssence"
    }
}


class sendTypeMode:
    # 发送者目标
    def __init__(self, session, uri):
        self._session = session
        self._url = uri
        self._c = Console()

    def __repr__(self):
        return "请选择一个信息执行模式"

    def friend(self, target: int):
        return friendTypeMode(session=self._session, url=self._url, target=target)

    def group(self, target: int):
        return groupTypeMode(session=self._session, target=target, url=self._url)

    def temp(self, gid: int):
        return tempTypeMode(session=self._session, gid=gid, url=self._url)

    def nudge(self, target: int):
        return expandNudge(self._url, self._session, target)

    def recall(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": target
        }
        data = po.post(self._url + str(_config["send"]["recall"]), data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：撤回成功",
                            "详细：" + str(target) + "(get) <- '撤回'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：撤回失败", style="#ff8f8f")
        return echoTypeMode(data)


class friendTypeMode:
    # 好友发送消息类型
    def __init__(self, session, url: str, target: int):
        self._session = session
        self._url = url
        self._target = target
        self._c = Console()

    def plain(self, context: str = " "):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {"type": "Plain", "text": context},
            ]
        }
        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：文本发送成功", "详细：" + str(self._target) + "(Friend) <- '" + str(context) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：文本发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def image(self, path: str):
        rec = re.compile(r'[a-zA-z]+://[^\s]*')
        if re.search(rec, path):
            data = {
                "sessionKey": self._session,
                "target": self._target,
                "messageChain": [
                    {"type": "Image", "url": path},
                ]
            }
        else:
            data = {
                "sessionKey": self._session,
                "target": self._target,
                "messageChain": [
                    {
                        "type": "Image",
                        "url": _uploadImage(path, self._session, "friend", self._url)
                    }
                ]
            }
        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
        if data["code"] == 0:
            self._c.log("[Notice]：图片发送成功", "详细：" + str(self._target) + "(Friend) <- '" + str(path) + "'",
                        style="#a4ff8f")
        else:
            self._c.log("[Error]：图片发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def face(self, faceId: int, name: str = ""):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "Face",
                    "faceId": faceId,
                    "name": name
                }
            ]
        }
        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：表情发送成功", "详细：" + str(self._target) + "(Friend) <- '" + str(faceId) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：表情发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def voice(self, voiceId: str = "", url: str = "", length: int = 1024):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "Voice",
                    "voiceId": voiceId,
                    "url": url,
                    "path": "null",
                    "base64": "null",
                    "length": length,
                }
            ]
        }

        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
        if data["code"] == 0:
            self._c.log("[Notice]：语音发送成功", "详细：" + str(self._target) + "(Friend) <- '" + str(voiceId) + "'",
                        style="#a4ff8f")
        else:
            self._c.log("[Error]：语音发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def poke(self):
        return expandPoke(self._url, self._session, self._target)

    def dice(self, value: int = 1):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "Dice",
                    "value": value
                }
            ]
        }
        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：骰子发送成功", "详细：" + str(self._target) + "(Friend) <- '未知'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：骰子发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def musicShare(self, value: int = 1):
        # 待解决
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "MusicShare",
                    "kind": "String",
                    "title": "String",
                    "summary": "String",
                    "jumpUrl": "String",
                    "pictureUrl": "String",
                    "musicUrl": "String",
                    "brief": "String"
                }
            ]
        }
        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：骰子发送成功", "详细：" + str(self._target) + "(Friend) <- '未知'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：骰子发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def app(self, content: str):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "App",
                    "content": content
                }
            ]
        }
        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：APP卡片发送成功", "详细：" + str(self._target) + "(Friend) <- 'APP卡片'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：APP卡片发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def other(self, data: dict):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                data
            ]
        }
        data = po.post(self._url + _config["send"]["friend"]["sendFriendMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：信息发送成功", "详细：" + str(self._target) + "(Friend) <- '未知'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：信息发送失败", style="#ff8f8f")
        return echoTypeMode(data)


class groupTypeMode:
    def __init__(self, session: str, url: str, target: int):
        self._url = url
        self._session = session
        self._c = Console()
        self._target = target

    # 发送群消息类型
    def at(self, target: int, display: str = ""):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "At",
                    "target": target,
                    "display": display
                }
            ]
        }
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)

            if data["code"] == 0:
                self._c.log("[Notice]：At发送成功", "详细：" + str(self._target) + "(Group) <- 'at " + str(target) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：At发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def atAll(self):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "AtAll"
                }
            ]
        }
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)

            if data["code"] == 0:
                self._c.log("[Notice]：AtAll发送成功",
                            "详细：" + str(self._target) + "(Group) <- 'at " + str(self._target) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：AtAll发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def plain(self, context: str):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {"type": "Plain", "text": context},
            ]
        }
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：文本发送成功",
                            "详细：" + str(self._target) + "(Group " + str(self._target) + ") <- '" + str(context) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：文本发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def image(self, path: str):
        rec = re.compile(r'[a-zA-z]+://[^\s]*')
        if re.search(rec, path):
            data = {
                "sessionKey": self._session,
                "target": self._target,
                "messageChain": [
                    {"type": "Image", "url": path},
                ]
            }
        else:
            data = {
                "sessionKey": self._session,
                "target": self._target,
                "messageChain": [
                    {
                        "type": "Image",
                        "url": _uploadImage(path, self._session, "group", self._url)
                    }
                ]
            }
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
        if data["code"] == 0:
            self._c.log("[Notice]：图片发送成功",
                        "详细：" + str(self._target) + "(Group " + str(self._target) + ") <- '" + str(path) + "'",
                        style="#a4ff8f")
        else:
            self._c.log("[Error]：图片发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def face(self, faceId: int, name: str = ""):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "Face",
                    "faceId": faceId,
                    "name": name
                }
            ]
        }
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：表情发送成功",
                            "详细：" + str(self._target) + "(Group " + str(self._target) + ") <- '" + str(faceId) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：表情发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def voice(self, voiceId: str = "", url: str = "", length: int = 1024):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "Voice",
                    "voiceId": voiceId,
                    "url": url,
                    "path": "null",
                    "base64": "null",
                    "length": length,
                }
            ]
        }

        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
        if data["code"] == 0:
            self._c.log("[Notice]：语音发送成功",
                        "详细：" + str(self._target) + "(Group " + str(self._target) + ") <- '" + str(voiceId) + "'",
                        style="#a4ff8f")
        else:
            self._c.log("[Error]：语音发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def poke(self):
        return expandPoke(self._url, self._session, self._target)

    def dice(self, value: int = 1):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "Dice",
                    "value": value
                }
            ]
        }
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：骰子发送成功",
                            "详细：" + str(self._target) + "(Group " + str(self._target) + ") <- '" + str(value) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：骰子发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def app(self, content: str):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "App",
                    "content": content
                }
            ]
        }
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：APP卡片发送成功",
                            "详细：" + str(self._target) + "(Group " + str(self._target) + ") <- 'APP卡片'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：APP卡片发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def other(self, data: dict):
        data = po.post(self._url + _config["send"]["group"]["sendGroupMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：信息发送成功", "详细：" + str(self._target) + "(Group) <- '未知'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：信息发送失败", style="#ff8f8f")
        return echoTypeMode(data)


class getTypeMode:
    def __init__(self, session: str, uri: str):
        self._url = uri
        self._session = session
        self._c = Console()

    def _get(self, message: str):
        data = po.get(self._url + str(_config["get"]["info"]))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：获取成功",
                            "详细：" + message + "(get) <- '获取'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：获取失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def info(self):
        return self._get("插件信息")

    @property
    def message(self):
        return expandGetMessage(self._url, self._session)

    @property
    def list(self):
        return expandGetList(self._url, self._session)

    @property
    def proFile(self):
        return expandGetProFile(self._url, self._session)

    def groupConfig(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": target
        }
        data = po.get(self._url + _config["get"]["groupConfig"], params=data)
        if data.status_code == 200:
            data = json.loads(data.text)
            if "code" not in data:
                self._c.log("[Notice]：获取成功",
                            "详细：获取群设置(get) <- " + str(target),
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：获取失败", style="#ff8f8f")
        return echoTypeMode(data)


class setTypeMode:
    def __init__(self, session: str, uri: str):
        self._url = uri
        self._session = session
        self._c = Console()

    def essence(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": target,
        }
        data = po.post(self._url + _config["set"]["essence"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：设置成功",
                            "详细：群精华消息(set) <- " + str(target),
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：设置失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def message(self):
        return expandGetMessage(self._url, self._session)

    @property
    def list(self):
        return expandGetList(self._url, self._session)

    @property
    def proFile(self):
        return expandGetProFile(self._url, self._session)

    def group(self, target: int):
        # 修改群相关设置
        return expandSetGroupConfig(self._url, self._session, target)


class tempTypeMode:
    def __init__(self, session: str, url: str, gid: int):
        self._url = url
        self._session = session
        self._c = Console()
        self._gid = gid

    # 发送群临时消息类型
    def to(self, target: int):
        return expandTemp(self._url, self._session, target, self._gid)


class uploadTypeMode:
    # 上传模式
    def __init__(self, session, uri):
        self._session = session
        self._url = uri

    def __repr__(self):
        return "请选择上传模式"

    def image(self, path: str):
        return expandUploadImage(self._url, self._session, path)


class actionTypeMode:
    # 操作模式
    def __init__(self, session, uri):
        self._session = session
        self._url = uri

    def __repr__(self):
        return "请选择群操作模式"

    def group(self, target: int):
        return expandActionGroup(self._url, self._session, target)

    @property
    def friend(self):
        return expandActionFriend(self._url, self._session)


class eventTypeMode:
    # 操作模式
    def __init__(self, session, uri, eventId):
        self._session = session
        self._url = uri
        self._eventId = eventId  # 事件ID

    def __repr__(self):
        return "请选择事件处理类型"

    def newFriend(self, target: int, groupId: int = 0):
        return expandEventNewFriend(self._url, self._session, target, self._eventId, groupId)

    def newJoinGroup(self, target: int, groupId: int):
        return expandEventJoinGroup(self._url, self._session, target, self._eventId, groupId)

    def newBotJoinGroup(self, target: int, groupId: int, fromId: int):
        return expandEventBotJoinGroup(self._url, self._session, target, self._eventId, groupId, fromId)


class echoTypeMode:

    def __init__(self, context):
        self._context = context

    def __repr__(self):
        return "<Request [OK]>"

    @property
    def json(self) -> str:
        """
        将返回值格式化为字符串类型
        @return: str
        """
        data = json.dumps(self._context)
        return data

    @property
    def dictionary(self) -> dict:
        """
        将返回值格式化为字典类型
        @return: dict
        """
        return self._context


def _uploadImage(path: str, session, name, url: str):
    message = {"sessionKey": session, "type": name}
    onfiles = {'img': ('send.png', open(path, 'rb'), 'image/png', {})}
    data = po.request(method="POST", url=url + "/uploadImage", data=message, files=onfiles)
    if data.status_code == 200:
        data = json.loads(data.text)
        if "code" not in data:
            return data["url"]
        else:
            return "error"



import re
from rich.console import Console

import requests as po

import json


_config = {
    "send": {
        "friend": {
            "sendFriendMessage": "/sendFriendMessage"
        },
        "group": {
            "sendGroupMessage": "/sendGroupMessage"
        },
        "temp": {
            "sendTempMessage": "/sendTempMessage"
        },
        "nudge": {
            "sendNudge": "/sendNudge"
        }
    },
    "action": {
        "mute": "/mute",
        "unmute": "/unmute",
        "muteAll": "/muteAll",
        "unmuteAll": "/unmuteAll",
        "kick": "/kick",
        "quit": "/quit",
        "deleteFriend": "/deleteFriend",
    },
    "event": {
        "newFriend": "/resp/newFriendRequestEvent",
    },
    "upload": {
        "uploadImage": "/uploadImage",
    },
    "get": {
        "count": "/countMessage",
        "fetchMessage": "/fetchMessage",
        "fetchLatestMessage": "/fetchLatestMessage",
        "peekMessage": "/peekMessage",
        "peekLatestMessage": "/peekLatestMessage",
        "messageFromId": "/messageFromId",
        "friendList": "/friendList",
        "groupList": "/groupList",
        "memberList": "/memberList",
        "friendProfile": "/friendProfile",
        "botProfile": "/botProfile",
        "memberProfile": "/memberProfile",
        "userProfile": "/userProfile",
        "groupConfig": "/groupConfig",
    }
}


class expandPoke:
    def __init__(self, url, session, target):
        self._url = url
        self._session = session
        self._target = target
        self._c = Console()

    def _context(self, name: str, to: str):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "messageChain": [
                {
                    "type": "Poke",
                    "name": name
                }
            ]
        }
        data = po.post(self._url + _config["send"]["friend"][to], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：戳一戳发送成功", "详细：" + str(self._target) + "(Friend) <- '" + name + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：戳一戳发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def poke(self):
        return self._context("poke", "sendFriendMessage")

    @property
    def FangDaZhao(self):
        return self._context("FangDaZhao", "sendFriendMessage")


class expandNudge:
    def __init__(self, url, session, target):
        self._url = url
        self._session = session
        self._target = target
        self._c = Console()

    def _context(self, name: str, by: str, to: str, subject: int):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "subject": subject,
            "kind": name
        }
        data = po.post(self._url + _config["send"][by][to], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：捏捏发送成功",
                            "详细：" + str(self._target) + "(" + name + " " + str(subject) + ") <- '捏捏'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：捏捏发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def friend(self):
        return self._context("Friend", "nudge", "sendNudge", self._target)

    def group(self, gid: int):
        return self._context("Group", "nudge", "sendNudge", gid)


class expandTemp:
    def __init__(self, url, session, target, gid):
        self._url = url
        self._session = session
        self._target = target
        self._gid = gid
        self._c = Console()

    def plain(self, context: str):
        data = {
            "sessionKey": self._session,
            "qq": self._target,
            "group": self._gid,
            "messageChain": [
                {"type": "Plain", "text": context},
            ]
        }
        print(data)
        data = po.post(self._url + _config["send"]["temp"]["sendTempMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：文本发送成功",
                            "详细：" + str(self._target) + "(Temp " + str(self._gid) + ") <- '" + str(context) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：文本发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def image(self, path: str):
        rec = re.compile(r'[a-zA-z]+://[^\s]*')
        if re.search(rec, path):
            data = {
                "sessionKey": self._session,
                "qq": self._target,
                "group": self._gid,
                "messageChain": [
                    {"type": "Image", "url": path},
                ]
            }
        else:
            data = {
                "sessionKey": self._session,
                "qq": self._target,
                "group": self._gid,
                "messageChain": [
                    {
                        "type": "Image",
                        "url": _uploadImage(path, self._session, "group", self._url)
                    }
                ]
            }
        data = po.post(self._url + _config["send"]["temp"]["sendTempMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
        if data["code"] == 0:
            self._c.log("[Notice]：图片发送成功",
                        "详细：" + str(self._target) + "(Temp " + str(self._gid) + ") <- '" + str(path) + "'",
                        style="#a4ff8f")
        else:
            self._c.log("[Error]：图片发送失败", style="#ff8f8f")
        return echoTypeMode(data)

    def face(self, faceId: int, name: str = ""):
        data = {
            "sessionKey": self._session,
            "qq": self._target,
            "group": self._gid,
            "messageChain": [
                {
                    "type": "Face",
                    "faceId": faceId,
                    "name": name
                }
            ]
        }
        data = po.post(self._url + _config["send"]["temp"]["sendTempMessage"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：表情发送成功",
                            "详细：" + str(self._target) + "(Group " + str(self._gid) + ") <- '" + str(faceId) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：表情发送失败", style="#ff8f8f")
        return echoTypeMode(data)


class expandGetMessage:
    def __init__(self, url, session):
        self._url = url
        self._session = session
        self._c = Console()

    def _request(self, message, to: str, params: dict):
        data = po.get(self._url + str(_config["get"][to]), params=params)
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：获取成功",
                            "详细：" + message + "(get) <- '获取信息'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：获取失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def count(self):
        data = {
            "sessionKey": self._session
        }
        return self._request("count", "count", data)

    def fetch(self, count: int):
        data = {
            "sessionKey": self._session,
            "count": count
        }
        return self._request("fetchMessage", "fetchMessage", data)

    def fetchLatest(self, count: int):
        data = {
            "sessionKey": self._session,
            "count": count
        }
        return self._request("fetchLatestMessage", "fetchLatestMessage", data)

    def peek(self, count: int):
        data = {
            "sessionKey": self._session,
            "count": count
        }
        return self._request("peekMessage", "peekMessage", data)

    def peekLatest(self, count: int):
        data = {
            "sessionKey": self._session,
            "count": count
        }
        return self._request("peekLatestMessage", "peekLatestMessage", data)

    def fromId(self, mid: int):
        data = {
            "sessionKey": self._session,
            "id": mid
        }
        return self._request("messageFromId", "messageFromId", data)


class expandGetList:
    def __init__(self, url, session):
        self._url = url
        self._session = session
        self._c = Console()

    def _request(self, message, to: str, params: dict):
        data = po.get(self._url + str(_config["get"][to]), params=params)
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：获取成功",
                            "详细：" + message + "(get) <- '获取'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：获取失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def friend(self):
        data = {
            "sessionKey": self._session
        }
        return self._request("friendList", "friendList", data)

    @property
    def group(self):
        data = {
            "sessionKey": self._session
        }
        return self._request("groupList", "groupList", data)

    def member(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": target
        }
        return self._request("memberList", "memberList", data)


class expandGetProFile:
    def __init__(self, url, session):
        self._url = url
        self._session = session
        self._c = Console()

    def _request(self, message, to: str, params: dict):
        data = po.get(self._url + str(_config["get"][to]), params=params)
        if data.status_code == 200:
            data = json.loads(data.text)
            if "code" not in data:
                self._c.log("[Notice]：获取成功",
                            "详细：" + message + "(get) <- '获取资料页'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：获取失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def bot(self):
        data = {
            "sessionKey": self._session
        }
        return self._request("botProfile", "botProfile", data)

    def friend(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": target
        }
        return self._request("friendProfile", "friendProfile", data)

    def member(self, gid: int, target: int):
        data = {
            "sessionKey": self._session,
            "target": gid,
            "memberId": target,
        }
        return self._request("memberProfile", "memberProfile", data)

    def user(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": target,
        }
        return self._request("userProfile", "userProfile", data)


class expandSetGroupConfig:
    # 修改群设置
    def __init__(self, url, session, target: int):
        self._url = url
        self._session = session
        self._target = target
        self._c = Console()

    def name(self, name: str):
        # 修改群名称
        data = _getGroupConfig(self._url, self._session, self._target)
        if "announcement" not in data:
            data["announcement"] = ""
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "config": {
                "name": name,
                "announcement": data["announcement"],
                "confessTalk": data["confessTalk"],
                "allowMemberInvite": data["allowMemberInvite"],
                "autoApprove": data["autoApprove"],
                "anonymousChat": data["anonymousChat"]
            }
        }
        data = po.post(self._url + _config["get"]["groupConfig"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：群配置修改成功",
                            "详细：" + str(self._target) + "(name) <- '" + str(name) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：群配置修改失败", style="#ff8f8f")
        return echoTypeMode(data)

    def announcement(self, content: str):
        # 修改群名称
        data = _getGroupConfig(self._url, self._session, self._target)
        if "announcement" not in data:
            data["announcement"] = ""
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "config": {
                "name": data["name"],
                "announcement": content,
                "confessTalk": data["confessTalk"],
                "allowMemberInvite": data["allowMemberInvite"],
                "autoApprove": data["autoApprove"],
                "anonymousChat": data["anonymousChat"]
            }
        }
        data = po.post(self._url + _config["get"]["groupConfig"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：群配置修改成功",
                            "详细：" + str(self._target) + "(announcement) <- '" + str(content) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：群配置修改失败", style="#ff8f8f")
        return echoTypeMode(data)

    # todo 完成群配置设置函数 还差4个选项
    def confessTalk(self, mode: bool):
        data = _getGroupConfig(self._url, self._session, self._target)
        if "announcement" not in data:
            data["announcement"] = ""
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "config": {
                "name": data["name"],
                "announcement": data["announcement"],
                "confessTalk": mode,
                "allowMemberInvite": data["allowMemberInvite"],
                "autoApprove": data["autoApprove"],
                "anonymousChat": data["anonymousChat"]
            }
        }
        data = po.post(self._url + _config["get"]["groupConfig"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：群配置修改成功",
                            "详细：" + str(self._target) + "(confessTalk) <- '" + str(mode) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：群配置修改失败", style="#ff8f8f")
        return echoTypeMode(data)

    def allowMemberInvite(self, mode: bool):
        data = _getGroupConfig(self._url, self._session, self._target)
        if "announcement" not in data:
            data["announcement"] = ""
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "config": {
                "name": data["name"],
                "announcement": data["announcement"],
                "confessTalk": data["confessTalkt"],
                "allowMemberInvite": mode,
                "autoApprove": data["autoApprove"],
                "anonymousChat": data["anonymousChat"]
            }
        }
        data = po.post(self._url + _config["get"]["groupConfig"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：群配置修改成功",
                            "详细：" + str(self._target) + "(allowMemberInvite) <- '" + str(mode) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：群配置修改失败", style="#ff8f8f")
        return echoTypeMode(data)

    def autoApprove(self, mode: bool):
        data = _getGroupConfig(self._url, self._session, self._target)
        if "announcement" not in data:
            data["announcement"] = ""
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "config": {
                "name": data["name"],
                "announcement": data["announcement"],
                "confessTalk": data["confessTalkt"],
                "allowMemberInvite": data["allowMemberInvite"],
                "autoApprove": mode,
                "anonymousChat": data["anonymousChat"]
            }
        }
        data = po.post(self._url + _config["get"]["groupConfig"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：群配置修改成功",
                            "详细：" + str(self._target) + "(autoApprove) <- '" + str(mode) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：群配置修改失败", style="#ff8f8f")
        return echoTypeMode(data)

    def anonymousChat(self, mode: bool):
        data = _getGroupConfig(self._url, self._session, self._target)
        if "announcement" not in data:
            data["announcement"] = ""
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "config": {
                "name": data["name"],
                "announcement": data["announcement"],
                "confessTalk": data["confessTalkt"],
                "allowMemberInvite": data["allowMemberInvite"],
                "autoApprove": data["autoApprove"],
                "anonymousChat": mode
            }
        }
        data = po.post(self._url + _config["get"]["groupConfig"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：群配置修改成功",
                            "详细：" + str(self._target) + "(anonymousChat) <- '" + str(mode) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：群配置修改失败", style="#ff8f8f")
        return echoTypeMode(data)


class expandEventNewFriend:
    def __init__(self, url, session, target, eventId, groupId):
        self._url = url
        self._session = session
        self._target = target
        self._eventId = eventId
        self._groupId = groupId
        self._c = Console()

    def _request(self, code: int, eventId, message):
        data = {
            "sessionKey": self._session,
            "eventId": self._eventId,
            "fromId": eventId,
            "groupId": self._groupId,
            "operate": code,
            "message": message
        }
        data = po.post(self._url + _config["event"]["newFriend"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：好友添加事件处理成功",
                            "详细：" + str(self._target) + "(Friend) <- '" + str(code) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：好友添加事件处理失败", style="#ff8f8f")
        return echoTypeMode(data)

    def yes(self, message: str = ""):
        return self._request(1, self._eventId, message)

    def no(self, message: str = ""):
        return self._request(0, self._eventId, message)

    def black(self, message: str = ""):
        return self._request(2, self._eventId, message)


class expandEventJoinGroup:
    def __init__(self, url, session, target, eventId, groupId):
        self._url = url
        self._session = session
        self._target = target
        self._eventId = eventId
        self._groupId = groupId
        self._c = Console()

    def _request(self, code: int, eventId, message):
        data = {
            "sessionKey": self._session,
            "eventId": self._eventId,
            "fromId": eventId,
            "groupId": self._groupId,
            "operate": code,
            "message": message
        }
        data = po.post(self._url + _config["event"]["newFriend"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：用户入群申请事件处理成功",
                            "详细：" + str(self._target) + "(Friend) <- '" + str(code) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：用户入群申请事件处理失败", style="#ff8f8f")
        return echoTypeMode(data)

    def yes(self, message: str = ""):
        return self._request(1, self._eventId, message)

    def no(self, message: str = ""):
        return self._request(0, self._eventId, message)

    def over(self, message: str = ""):
        return self._request(2, self._eventId, message)

    def noBlack(self, message: str = ""):
        return self._request(3, self._eventId, message)

    def overBlack(self, message: str = ""):
        return self._request(4, self._eventId, message)


class expandEventBotJoinGroup:
    def __init__(self, url, session, target, eventId, groupId, fromId):
        self._url = url
        self._session = session
        self._target = target
        self._eventId = eventId
        self._groupId = groupId
        self._fromId = fromId
        self._c = Console()

    def _request(self, code: int, message):
        data = {
            "sessionKey": self._session,
            "eventId": self._eventId,
            "fromId": self._fromId,
            "groupId": self._groupId,
            "operate": code,
            "message": message
        }
        data = po.get(self._url + _config["event"]["newFriend"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：用户入群申请事件处理成功",
                            "详细：" + str(self._target) + "(Friend) <- '" + str(code) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：用户入群申请事件处理失败", style="#ff8f8f")
        return echoTypeMode(data)

    def yes(self, message: str = ""):
        return self._request(1, message)

    def no(self, message: str = ""):
        return self._request(0, message)


class expandUploadImage:
    def __init__(self, url, session, path):
        self._url = url
        self._session = session
        self._path = path
        self._c = Console()

    def _context(self, name: str, subject: int):
        message = {"sessionKey": self._session, "type": name}
        onfiles = {'img': ('send.png', open(subject, 'rb'), 'image/png', {})}
        data = po.request(method="POST",
                          url=self._url + "/uploadImage", data=message, files=onfiles)
        if data.status_code == 200:
            data = json.loads(data.text)
            if "code" not in data:
                self._c.log("[Notice]：图片上传成功",
                            "详细：Cloud(" + name + " " + str(subject) + ") <- '" + self._path + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：图片上传失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def friend(self):
        return self._context("friend", self._path)

    @property
    def group(self):
        return self._context("group", self._path)

    @property
    def temp(self):
        return self._context("temp", self._path)


class expandActionGroup:
    def __init__(self, url, session, gid):
        self._url = url
        self._session = session
        self._target = gid
        self._c = Console()

    def mute(self, target: int):
        return expandActionGroupMute(self._url, self._session, self._target, target)

    def unmute(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "memberId": target,
        }
        data = po.post(self._url + _config["action"]["unmute"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)

            if data["code"] == 0:
                self._c.log("[Notice]：解除禁言成功",
                            "详细：" + str(self._target) + "(Group) <- '" + str(target) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：解除禁言失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def muteAll(self):
        data = {
            "sessionKey": self._session,
            "target": self._target,
        }
        data = po.post(self._url + _config["action"]["muteAll"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)

            if data["code"] == 0:
                self._c.log("[Notice]：全体禁言成功",
                            "详细：" + str(self._target) + "(Group) <- 'muteAll'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：全体禁言失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def unMuteAll(self):
        data = {
            "sessionKey": self._session,
            "target": self._target,
        }
        data = po.post(self._url + _config["action"]["unmuteAll"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)

            if data["code"] == 0:
                self._c.log("[Notice]：全体禁言成功",
                            "详细：" + str(self._target) + "(Group) <- 'unMuteAll'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：全体禁言失败", style="#ff8f8f")
        return echoTypeMode(data)

    def kick(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "memberId": target,
            "msg": "您已被移出群聊"
        }
        data = po.post(self._url + _config["action"]["kick"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：移除群成员成功",
                            "详细：" + str(self._target) + "(Group) <- '" + str(target) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：移除群成员失败", style="#ff8f8f")
        return echoTypeMode(data)

    @property
    def quit(self):
        data = {
            "sessionKey": self._session,
            "target": self._target,
        }
        data = po.post(self._url + _config["action"]["quit"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：退出群聊成功",
                            "详细：" + str(self._target) + "(Group) <- 'quit'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：退出群聊失败", style="#ff8f8f")
        return echoTypeMode(data)


class expandActionFriend:
    def __init__(self, url, session):
        self._url = url
        self._session = session
        self._c = Console()

    def deleteFriend(self, target: int):
        data = {
            "sessionKey": self._session,
            "target": target,
        }
        data = po.post(self._url + _config["action"]["deleteFriend"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：移除好友成功",
                            "详细：" + str(target) + "(Friend) <- 'deleteFriend'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：移除好友失败", style="#ff8f8f")
        return echoTypeMode(data)


class expandActionGroupMute:
    def __init__(self, url, session, target, memberId):
        self._url = url
        self._session = session
        self._target = target
        self._memberId = memberId
        self._c = Console()

    def _request(self, time: int):
        if time >= 2592000:
            time = 2591999
        data = {
            "sessionKey": self._session,
            "target": self._target,
            "memberId": self._memberId,
            "time": time
        }
        data = po.post(self._url + _config["action"]["mute"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：禁言成功",
                            "详细：" + str(self._target) + "(Group) <- '" + str(time) + " s " + str(self._memberId) + "'",
                            style="#a4ff8f")
            else:
                self._c.log("[Error]：禁言失败", style="#ff8f8f")
        return echoTypeMode(data)

    def s(self, second: int):
        return self._request(second)

    def m(self, minute: int):
        second = minute * 60
        return self._request(second)

    def h(self, minute: int):
        second = minute * 60 * 60
        return self._request(second)

    def d(self, day: int):
        second = day * 60 * 24 * 60
        return self._request(second)


def _uploadImage(path: str, session, name, url: str):
    message = {"sessionKey": session, "type": name}
    onfiles = {'img': ('send.png', open(path, 'rb'), 'image/png', {})}
    data = po.request(method="POST", url=url + "/uploadImage", data=message, files=onfiles)
    if data.status_code == 200:
        data = json.loads(data.text)
        if "code" not in data:
            return data["url"]
        else:
            return "error"


def _getGroupConfig(url, session, target: int):
    # 获取群配置信息
    message = {"sessionKey": session, "target": target}
    data = po.get(url=url + _config["get"]["groupConfig"], params=message)
    if data.status_code == 200:
        data = json.loads(data.text)
        if "code" not in data:
            return data
        else:
            return "error"



import json
from .models import (
    sendTypeMode,
    getTypeMode,
    setTypeMode,
    uploadTypeMode,
    actionTypeMode,
    eventTypeMode
)
from concurrent.futures import ThreadPoolExecutor
import atexit

import requests as req
from rich.console import Console

_config = {
    "about": "/about",
    "verify": "/verify",
    "bind": "/bind",
    "release": "/release",
    "data": {
        "url": "",
        "session": "",
        "botId": ""
    }
}


class Init:
    def __init__(self, host: str, port: str, botId: str, key: str, maxWork: int = 8):
        self._host: str = host
        self._port: str = port
        self._botId: str = botId
        self._key: str = key
        self._session: str = ""
        self._c = Console()
        self._url = self._host + ":" + self._port  # 格式化Bot HTTP API地址
        self._testServer()
        self._pool = self._createPool(maxWork)  # 创建线程池
        self._beginSession()
        _config["data"]["url"] = self._url
        _config["data"]["botId"] = self._botId
        _config["data"]["session"] = self._session

    def _testServer(self):
        # 测试服务器是否正常开启
        try:
            request = req.get(url=self._url + _config["about"], timeout=5)
        except Exception as re:
            request = re
            self._c.log("[Alert]：地址或端口有误，详细：未查询到Mirai HTTP服务器", style="#fb48a0")
            exit(404)
        if request.status_code == 200:
            data = json.loads(request.text)
            if data["code"] == 0:
                version = data["data"]["version"]
                self._c.log("[Notice]：当前Mirai-HTTP-API版本为 " + version + " ，详细: 已查询到Mirai HTTP服务器", style="#a4ff8f")

    def _createPool(self, maxWork: int):
        pool = ThreadPoolExecutor(maxWork)
        self._c.log("[Notice]：成功创建容量为 " + str(maxWork) + " 的线程池", style="#a4ff8f")
        return pool

    def _beginSession(self):
        data = {
            "verifyKey": self._key
        }
        data = req.post(url=self._url + _config["verify"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._session = data["session"]
            else:
                self._c.log("[Error]：获取Session失败", style="#ff8f8f")

        data = {
            "sessionKey": self._session,
            "qq": self._botId
        }
        data = req.post(url=self._url + _config["bind"], data=json.dumps(data))
        if data.status_code == 200:
            data = json.loads(data.text)
            if data["code"] == 0:
                self._c.log("[Notice]：绑定Session成功", "详细：session=" + self._session, style="#a4ff8f")
            else:
                self._c.log("[Error]：绑定Session失败", style="#ff8f8f")


class Mirai(Init):

    def __repr__(self):
        return "请选择一个操作模式"

    # 开放接口
    @property
    def send(self):
        return sendTypeMode(session=self._session, uri=str(self._url))

    @property
    def get(self):
        return getTypeMode(session=self._session, uri=str(self._url))

    @property
    def set(self):
        return setTypeMode(session=self._session, uri=str(self._url))

    @property
    def upload(self):
        return uploadTypeMode(session=self._session, uri=str(self._url))

    @property
    def action(self):
        return actionTypeMode(session=self._session, uri=str(self._url))

    def event(self, eventId: int):
        return eventTypeMode(session=self._session, uri=str(self._url), eventId=eventId)

    # todo 添加File操作类


@atexit.register
def _stop():
    _c = Console()
    data = {
        "sessionKey": _config["data"]["session"],
        "qq": _config["data"]["botId"]
    }
    try:
        data = req.post(url=_config["data"]["url"] + _config["release"], data=json.dumps(data))
    except Exception as re:
        requests = re
        _c.log("[Alert]：地址或端口有误，详细：未查询到Mirai HTTP服务器，无法注销Session", style="#fb48a0")
        exit(404)
    if data.status_code == 200:
        data = json.loads(data.text)
        if data["code"] == 0:
            _c.log("[Notice]：释放Session成功", "详细：session=" + _config["data"]["session"], style="#a4ff8f")
        else:
            _c.log("[Error]：释放Session失败", style="#ff8f8f")
